!***********************************************************************
!
!                     BASIN_SOILMOIST__DPM.F  
!
!  CALCULATES: CURRENT SOIL MOISTURES AND SATURATIONS FOR BASIN AND CELLS
!                  -     -      -     -
!     Declares, initializes, and then CALCULATES FOR each day
!     THE VALUES FOR THE MODELED HRUS/CELLS
! 
!....SUBROUTINE TO CALCULATE CURRENT SOIL MOISTURES AND SATURATIONS:
!....MIN, MAX, AREA WEIGHTED AVERAGE, AND CURRENT FOR ALL CELLS.
!....UNSATURATED MOISTURE IN LAYERS BELOW THE ROOT ZONE IS "INACTIVE".

!    OUTPUT CAN BE DONE THROUGH USE OF MMS STATVAR FILES

  
!     version: 1.0 (vaccaro/bauer)


      integer function basin_soilmoist_dpm(arg)

      include 'fmodules.inc'

      character*(*) arg

      integer basinsmdecl, basinsminit, basinsmrun, retval

      integer  nhru, nsoils
      integer  cov_type(MAXHRU), hru_soil(MAXHRU)

      real  wght(MAXHRU)
      real  smsavg,       satavg
      real  smscellmn, satcellmn, smscellmx, satcellmx
      real  defctsms,     defctsat
      real  smsdefct(MAXHRU), satdefct(MAXHRU)
      real  unsatmax(MAXHRU), stormax(MAXHRU)
      real  avlcap(MAXSOILS), spcyld(MAXSOILS), nlayer(MAXSOILS)

      save nhru, nsoils 
      save cov_type, wght, unsatmax, stormax
      save smsavg,  satavg
      save smscellmn, satcellmn, smscellmx, satcellmx
      save defctsms, defctsat, smsdefct, satdefct
      save avlcap, spcyld, nlayer, hru_soil

      retval = 0

      if(arg.eq.'declare') then
        retval = basinsmdecl(smsavg, satavg, smscellmn, satcellmn,
     +                                       smscellmx, satcellmx,
     +                        defctsms,defctsat,smsdefct, satdefct)
      
      else if(arg.eq.'initialize') then
        retval = basinsminit(nhru, nsoils, unsatmax, stormax, cov_type,
     +                          hru_soil, avlcap, spcyld, nlayer, wght)
     +                         

      else if(arg.eq.'run') then
        retval = basinsmrun(nhru,cov_type,wght,unsatmax,stormax,  
     +                                                smsavg, satavg,
     +                    smscellmn, satcellmn, smscellmx, satcellmx,
     +                           defctsms,defctsat,smsdefct, satdefct)

      end if

!******Debug level print

      call dpint4('End of basin_soilmoist, retval = ', retval, 1, 2)

      basin_soilmoist_dpm = retval

      return
      end

!***********************************************************************
!     basinsmdecl: Declares variables and parameters 
!                 for soil waters for modeled HRUs and whole basin
!              
!***********************************************************************


!***********************************************************************
 
!     basinsmdecl - set up parameters


      integer function basinsmdecl(smsavg,satavg,smscellmn, satcellmn,
     +                                           smscellmx, satcellmx,
     +                         defctsms, defctsat, smsdefct, satdefct) 

      include 'fmodules.inc'

      real  smsavg,    satavg
      real  smscellmn, satcellmn, smscellmx, satcellmx
      real  defctsms,         defctsat
      real  smsdefct(MAXHRU), satdefct(MAXHRU)

      basinsmdecl = 1

!..DECLARE THE VARIABLES FOR THIS MODULE
!...FIRST ARE THE CURRENT DEFICITS for SOIL AND SATURATED CONTENTS FOR EACH CELL/HRU

      if(declvar('basinsm', 'smsdefct', 'nhru', MAXHRU, 'real',
     +   'Deficit of water in a HRU unsaturated soil moisture store',
     +   'inches',smsdefct).ne.0) return

      if(declvar('basinsm', 'satdefct', 'nhru', MAXHRU, 'real',
     +   'Deficit of water in a HRU saturated soil moisture store',
     +   'inches',satdefct).ne.0) return


!..CURRENT AREA WEIGHTED AVERAGE OF SOIL MOSITURE AND SOIL SATURATION
!..CALCULATED FROM CURRENT VALUES FOR EACH HRU/CELL, AND BASIN-AVERAGE
!..DEFICITS IN THE UNSATURATED AND SATURATED STORES

      if(declvar('basinsm', 'smsavg', 'one', 1, 'real',
     +   'Area-average of soil moisture capacity for all CELLs/HRUs'//
     +   'based on CURRENT SOIL MOISTURE for each cell',
     +   'inches', smsavg).ne.0) return

      if(declvar('basinsm', 'satavg', 'one', 1, 'real',
     +   'Area-average of soil saturation capacity for all CELLs/HRUs'//
     +   'based on CURRENT SATURATED MOISTURE for each cell',
     +   'inches', satavg).ne.0) return

      if(declvar('basinsm', 'defctsms', 'one', 1, 'real',
     +   'Current deficit soil moisture capacity for all CELLs/HRUs'//
     +   'based on values for every hru',
     +   'inches', defctsms).ne.0) return

      if(declvar('basinsm', 'defctsat', 'one', 1, 'real',
     +   'Current deficit saturated moisture capacity for all HRUs'//
     +   'based on values for every hru',
     +   'inches', defctsat).ne.0) return


!..CURRENT MAXIMUM AND MINIMUM CELL/HRU VALUES FOR THE BASIN

      if(declvar('basinsm', 'smscellmn', 'one', 1, 'real',
     +   'Current minimum soil moisture capacity for all CELLs/HRUs'//
     +   'based on values for every hru',
     +   'inches', smscellmn).ne.0) return

      if(declvar('basinsm', 'smscellmx', 'one', 1, 'real',
     +   'Current maximum soil moisture capacity for all CELLs/HRUs'//
     +   'based on values for every hru',
     +   'inches', smscellmx).ne.0) return

      if(declvar('basinsm', 'satcellmn', 'one', 1, 'real',
     +   'Current minimum saturated moisture capacity for all HRUs'//
     +   'based on values for every hru',
     +   'inches', satcellmn).ne.0) return

      if(declvar('basinsm', 'satcellmx', 'one', 1, 'real',
     +   'Current maximum saturated moisture capacity for all HRUs'//
     +   'based on values for every hru',
     +   'inches', satcellmx).ne.0) return


!..SOIL PARAMETERS: MAXIMUM STORAGE IN THE UNSATURATED AND SATURATED SOIL MOISTURE STORES


      if(declparam('basinsm', 'unsatmax', 'nhru', 'real',
     +   '2.0', '0.', '24.',
     +   'Maximum available water capacity for a soil for a hru',
     +   'Maximum available water capacity for a soil for a hru',
     +   'inches').ne.0) return

      if(declparam('basinsm', 'stormax', 'nhru', 'real',
     +   '2.0', '0.', '24.',
     +   'Maximum saturated water capacity for a soil for a hru',
     +   'Maximum saturated water capacity for a soil for a hru',
     +   'inches').ne.0) return

      if(declparam('soilms', 'avlcap', 'nsoils', 'real',
     +   '0.05', '0.', '.7',
     +   'Available water capacity for a soil association',
     +   'Available water capacity for a soil association'//
     +   'as decimal fraction for the layers (6 inches thick)',
     +   'none').ne.0) return

      if(declparam('soilms', 'spcyld', 'nsoils', 'real',
     +   '0.20', '0.02', '.45',
     +   'Specific yield of soils for a soil association',
     +   'Specific yield of soils for a soil association, equivalent'//
     +   ' to tot effective porosity, decimal fraction by volume ',
     +   'none').ne.0) return

      if(declparam('soilms', 'nlayer', 'nsoils', 'integer',
     +   '4', '1', '14',
     +   'Number of 6" layers for a soil association',
     +   'Number of 6" layers for a soil association, will be limited'//
     +   'by the depth of the root zone during calculations',
     +   'none').ne.0) return

!..basin parameters

      if(declparam('basin', 'hru_soil', 'nhru', 'integer',
     +   '1', '1', '200',
     +   'HRU soil type',
     +   'HRU soil type',
     +   'none').ne.0) return

      if(declparam('basin', 'cov_type', 'nhru', 'integer',
     +   '1', '1', '31',
     +   'HRU cover type: land use/cover type, from 1-31',
     +   'HRU cover type, represents from 1-31 landuses/covers',
     +   'none').ne.0) return


      basinsmdecl = 0

      return
      end



!**********************************************************************
!     basinsminit - get input  parameters, calculate, and then initialize 
!               

      integer function basinsminit(nhru, nsoils, unsatmax, stormax, 
     +                cov_type,hru_soil,avlcap,spcyld, nlayer, wght)

      include 'fmodules.inc'

      integer nhru, nsoils
      integer cov_type(MAXHRU), hru_soil(MAXHRU), nlayer(MAXSOILS)

      integer i, ns, nltot
 
      real  wght(MAXHRU)
      real  unsatmax(MAXHRU), stormax(MAXHRU)
      real  avlcap(MAXSOILS), spcyld(MAXSOILS)

      basinsminit = 1

      nhru = getdim('nhru')
      if(nhru.eq.-1) return

      nsoils = getdim('nsoils')
      if(nsoils.eq.-1) return

      if(getvar('basin',    'wght',       MAXHRU,    'real',    wght)
     +  .ne.0) return

!      if(getparam('soilms', 'unsatmax',    MAXHRU,   'real', unsatmax)
!     +   .ne.0) return

!      if(getparam('soilms', 'stormax',     MAXHRU,   'real',  stormax)
!     +   .ne.0) return

!.. Get HRU/MRU soil information for initializing

      if(getparam('basin', 'hru_soil',   MAXHRU, 'integer', hru_soil)
     +  .ne.0) return

!..SOIL PARAMETERS

      if(getparam('soilms', 'avlcap',    MAXSOILS, 'real', avlcap)
     +   .ne.0) return

      if(getparam('soilms', 'spcyld',    MAXSOILS, 'real', spcyld)
     +    .ne.0) return

      if(getparam('soilms', 'nlayer',    MAXSOILS, 'integer', nlayer)
     +   .ne.0) return

!..landuse/cover type

      if(getparam('basin', 'cov_type',   MAXHRU,  'integer', cov_type)
     +   .ne.0) return

!..find max storages in root zone, excluding water bodies =cov_type=10
!..                                excluding impervious   =cov_type=16

      do 500 i=1,nhru

      if(cov_type(i).eq.10.or.cov_type(i).eq.16)      go to 500

!..  SOIL IDENTIFICATION NUMBER FOR THIS BLOCK
      NS=HRU_SOIL(I)
!..  NUMBER OF 6 IN LAYERS OF SOIL
      NLTOT=NLAYER(NS)
!...SOIL and ROOT DEPTH MUST BE AT LEAST 6 INCHES (ONE LAYER)
      IF(NLTOT.LT.1) NLTOT=1

!..  TOTAL AVAILABLE WATER CAPACITY FOR THE SOIL COLUMN, REGARDLESS OF ROOT DEPTH
!..  use avlcap that is decimal percent of a 6 layer
!..  TOTAL SATURATED WATER CONTENT FOR THE SOIL COLUMN, REGARDLESS OF ROOT DEPTH
!..  =EFFECTIVE POROSITY (SPEC. YIELD), IN INCHES, FOR TOTAL SOIL THICKNESS

      UNSATMAX(i)= AVLCAP(NS) * NLTOT*6.0
      STORMAX(i) = SPCYLD(NS) * NLTOT*6.0

500   continue

      basinsminit = 0

      return
      end

!***********************************************************************
! 
!     basinsmrun - Calculate  moisture stores for all cells and basin weighted averages
!                                                     and min,max cell values for basin       

      integer function basinsmrun(nhru,cov_type,wght,unsatmax,stormax, 
     +                             smsavg,satavg, smscellmn,satcellmn,
     +                                           smscellmx, satcellmx,
     +                           defctsms,defctsat,smsdefct, satdefct)


      include 'fmodules.inc'

!..INITIALIZED VARIABLES/PARAMETERS FROM BASINSMINIT

      integer nhru, cov_type(MAXHRU)
     
      real  wght(MAXHRU)
      real  unsatmax(MAXHRU), stormax(MAXHRU)

!..MODELED VARIABLES from other modules, not updated so no need to 'putvar'

      real  unsatcur(MAXHRU), storcur(MAXHRU)

!..VARIABLES FOR THIS MODULE

      real  smsavg,       satavg
      real  smscellmn, satcellmn, smscellmx, satcellmx
      real  defctsms,     defctsat
      real  smsdefct(MAXHRU), satdefct(MAXHRU)

!..temp variables/counter

      integer i
      real  smsblk, satblk

      BASINSMRUN = 1

!....RUN PART OF MODULE TO CALCULATE CURRENT SOIL MOISTURES AND SATURATIONS:
!....MIN, MAX, AND AREA WEIGHTED AVERAGE FOR BASIN
!..    AND CURRENT FOR CELLS.
!....UNSATURATED MOISTURE IN LAYERS BELOW THE ROOT ZONE IS "INACTIVE".

!.....INITIALIZE:

      SMSAVG   = 0.0
      SATAVG   = 0.0
      DEFCTSMS = 0.0
      DEFCTSAT = 0.0
      SMSCELLMN= 9999.
      SMSCELLMX= 0.0
      SATCELLMN= 9999.
      SATCELLMX= 0.0

!..GET THE CURRENT SOIL MOISTURE AND SATURATED MOISTURE

      if(getvar('soilms',  'unsatcur',   MAXHRU, 'real', unsatcur)
     +   .ne.0) return

      if(getvar('soilms',  'storcur',    MAXHRU, 'real', storcur)
     +   .ne.0) return

!.....ADD UP SOIL MOISTURE, AND SATURATION FOR HRUS/CELLS/BLOCKS AND
!.....GET AREA WEIGHTED AVERAGE. ALSO FIND MAX AND MIN BLOCK VALUES.


      DO 25 I=1,nhru
      SMSAVG=0.0
      SATAVG=0.0
      SMSBLK=0.0
      SATBLK=0.0
      DEFCTSMS = 0.0
      DEFCTSAT = 0.0
      smsdefct(i) = 0.0
      satdefct(i) = 0.0


!..if water or impervious then skip to next hru/mru/cell

      if(cov_type(i).eq.10 .or. cov_type(i).eq.16) go to 25

      SMSBLK = unsatcur(i)
      SATBLK = storcur(i)

!..CALCULATE CURRENT DEFICITS FOR THE SOIL MOISTURE STORES, ACCUM BASIN TOTALS

      smsdefct(i) = smsblk - unsatmax(i)
      satdefct(i) = satblk - stormax(i)
      DEFCTSMS = DEFCTSMS + SMSDEFCT(I)*WGHT(i)
      DEFCTSAT = DEFCTSAT + SATDEFCT(I)*WGHT(i)

!..FIND BASIN MINIMUM AND MAXIMUM SOIL MOISTURES FOR ANY CELL

      IF(SMSBLK.LT.SMSCELLMN) SMSCELLMN=SMSBLK
      IF(SMSBLK.GT.SMSCELLMX) SMSCELLMX=SMSBLK
      IF(SATBLK.LT.SATCELLMN) SATCELLMN=SATBLK
      IF(SATBLK.GT.SATCELLMX) SATCELLMX=SATBLK

!..ACCUMULATE WEIGHTED AVERAGE FOR BASIN OF SOIL MOISTURE IN THE STORES

      SMSAVG=SMSAVG+SMSBLK*WGHT(i)
      SATAVG=SATAVG+SATBLK*WGHT(i)

25    continue


      basinsmrun = 0

      RETURN
      END


