!***********************************************************************

!                     EXTRAD__DPM.F  

!            EXTRATERRESTRIAL DAILY SOLAR RADIATION

!.. calculate solar radiation under clear skys for horizontal and sloped surfaces
!.. calculations account for observed solar rad values that have been interpolated
!.. to the hrus/mrus. if no obs data then use calculated values


!...CALCULATE EXTRATERRESTRIAL DAILY SOLAR RADIATION, ASSUMING NO
!...ATMOSPHERE (SLREXTHZ), IN LANGLEYS, INCIDENT ON HORIZONTAL/SLOPED SURFACE
!...AND FRACTION OF DAY THAT SUN IS UP (FRCDYLT).
!...ASSUME THAT IT APPLYS FOR ALL HRUS when no data and as modifying factor when data.

!     version: 1.0 (vaccaro/bauer)


      integer function extrad_dpm(arg)

      include 'fmodules.inc'

      character*(*) arg

      integer extraddecl, extradinit, extradrun, retval

      integer nsol, nhru

      real slrexthz(MAXHRU), slrext(MAXHRU), frcdylt(MAXHRU)
      real hru_slope(MAXHRU), hru_aspect(MAXHRU), avelat

      save avelat, slrexthz, slrext, frcdylt, nsol, nhru
      save hru_slope, hru_aspect

      retval = 0

      if(arg.eq.'declare') then
        retval = extraddecl(slrexthz, slrext, frcdylt)

      
      else if(arg.eq.'initialize') then
        retval = extradinit(nsol, nhru, avelat,
     +                                    hru_slope,hru_aspect)

      else if(arg.eq.'run') then
        retval = extradrun(nsol, nhru, slrexthz, slrext, frcdylt,
     +                             avelat, hru_slope, hru_aspect)

      end if

!******Debug level print
      call dpint4('End of extrad: solar rad, retval = ', retval, 1, 2)

      extrad_dpm = retval

      return
      end

!***********************************************************************
!     extraddecl: Declares the daily solar radiation variable
!              
!***********************************************************************


!***********************************************************************
! 
!     extraddecl - set up parameters
!

      integer function extraddecl(slrexthz, slrext, frcdylt)


      include 'fmodules.inc'

      real slrexthz(MAXHRU), slrext(MAXHRU), frcdylt(MAXHRU)

      extraddecl = 1

!..Solar radiation for julian day of simulation: for horizontal surface

      if(declvar('extrad', 'slrexthz', 'nhru', MAXHRU, 'real',
     +   'Solar radiation for horizontal surface for simulation day'//
     +   ' calculated assuming no atmosphere',
     +   'langleys', slrexthz).ne.0) return

!..Solar radiation for julian day of simulation: for sloped surface

      if(declvar('extrad', 'slrext', 'nhru', MAXHRU, 'real',
     +   'Solar radiation for sloped surface for simulation day'//
     +   ' calculated assuming no atmosphere',
     +   'langleys', slrext).ne.0) return

!..FRACTION OF DAY THAT THE SUN IS UP, DAYLIGHT HOURS

      if(declvar('extrad', 'frcdylt', 'nhru', MAXHRU, 'real',
     +   'Fraction of day with sun--daylight',
     +   'fraction', frcdylt).ne.0) return

!..AVERAGE LATITUDE OF BASIN FOR CACULATING SOLAR RADIATION

      if(declparam('extrad', 'avelat', 'one', 'real',
     +   '30.', '20.', '60.',
     +   'Average latitude of basin used for solar rad calculations',
     +   'Average latitude of basin used for solar rad calculations '//
     +   'in decimal degrees',
     +   'degrees').ne.0) return

!..slope and aspect for tilted surface extraterristrial radition

      if(declparam('basin', 'hru_slope', 'nhru', 'real',
     +   '0.', '0.', '90.',
     +   'HRU slope in degrees from horizontal',
     +   'HRU slope in degrees from horizontal',
     +   'degrees (1-90)')
     +   .ne.0) return

      if(declparam('basin', 'hru_aspect', 'nhru', 'real',
     +   '0.', '0.', '360.',
     +   'HRU aspect in degrees from north: 0-360',
     +   'HRU aspect in degrees from north: 0-360',
     +   'degrees')
     +   .ne.0) return

      extraddecl = 0

      return
      end

!**********************************************************************
!     extradinit - get input  parameters, calculate,  and then initialize 
!               
           

      integer function extradinit(nsol, nhru, avelat,
     +                               hru_slope, hru_aspect)

      include 'fmodules.inc'

      integer nsol, nhru
      real avelat, hru_slope(MAXHRU), hru_aspect(MAXHRU)

      character*100 buffer

      extradinit = 1

      nsol = getdim('nsol')
      if(nsol.eq.-1) return

      nhru = getdim('nhru')
      if(nhru.eq.-1) return

      if(getparam('netrad', 'avelat',     1,     'real',     avelat)
     +  .ne.0) return

      if(getparam('basin', 'hru_slope',  MAXHRU, 'real',  hru_slope)
     +   .ne.0) return

      if(getparam('basin', 'hru_aspect', MAXHRU, 'real',  hru_aspect)
     +  .ne.0) return

      WRITE (BUFFER,1000)
      CALL OPSTR(BUFFER)
      WRITE (BUFFER,1001) AVELAT
      CALL OPSTR(BUFFER)

1000  FORMAT()
1001  FORMAT ('  AVERAGE LATITUDE OF STUDY AREA (DECIMAL.DEGREES) FOR PE  
     +T CALCULATIONS (AVELAT) =',F10.4)

      extradinit = 0

      return
      end

!***********************************************************************
 
!     extradrun - Calculate extraterrestrial solar radiation for cells


      integer function extradrun(nsol,nhru, slrexthz, slrext, frcdylt,
     +                                 avelat, hru_slope, hru_aspect)

      include 'fmodules.inc'

      integer nsol, nhru

!..MODELED VARIABLES

      real  slrexthz(MAXHRU),   slrext(MAXHRU),    frcdylt(MAXHRU)
      real hru_solrad(MAXHRU), hru_slope(MAXHRU), hru_aspect(MAXHRU)

!..Varaibles from grid_met_dpm.f

      real tminf(MAXHRU), tmaxf(MAXHRU)

!.. parameters
    
      real avelat
      real asun, bsun, pps

!..temporary values

      real ratio
      integer i, jday

      extradrun = 1

      jday   = julian('now', 'calendar')

      if(getvar('grid',   'hru_solrad',    MAXHRU, 'real', hru_solrad)
     +   .ne.0) return

      if(getvar('grid',     'tmaxf',        MAXHRU, 'real', tmaxf)
     +   .ne.0) return

      if(getvar('grid',     'tminf',        MAXHRU, 'real', tminf)
     +   .ne.0) return

      asun=-20.0
      bsun=3.636

      do 1000 i=1,nhru

!.. First check if observed solar radiation was interpolated to HRUs
!.. if so, and hru_slope is < 1.0 degrees no need to adjust for slope/aspect

      if(nsol.gt.0 .and. hru_slope(i).lt.1.0) go to 1000

!..Either no solar data or slope > 1.0.  Either way need to do following:
!..                                     compute solar rad on horizontal surface

      CALL EXTRAD(JDAY,AVELAT,0.0, 0.0, slrexthz(i), FRCDYLT(i))

!..NOW have solar rad on horizontal surface, calculate for sloped surface of hru, but only
!..if slope is gt.1.0 (can have observed data but slope >1.0
!..                    or no observed data but slope is < 1.0 or gt 1.0

      if(hru_slope(i).lt.1.0) THEN

         pps= (asun+bsun*(tmaxf(i)-tminf(i)))/100.

         if(pps.gt.1.0) pps=1.0
         if(pps.lt.0.0) pps=0.0

         hru_solrad(i) = slrexthz(i) * (0.23+0.48*pps)

      ELSE

!.. whether have observed data or not, need to adjust for slope
!.. calculate solrad for sloped surface


      CALL EXTRAD(JDAY,AVELAT,hru_slope(i),hru_aspect(i),
     +                            SLREXT(i),FRCDYLT(i))


        if(nsol.gt.0) then
         ratio = SLREXT(i)/SLREXTHZ(i)
         hru_solrad(i)=hru_solrad(i) * ratio
        else
         pps= (asun+bsun*(tmaxf(i)-tminf(i)))/100.

         if(pps.gt.1.0) pps=1.0
         if(pps.lt.0.0) pps=0.0

         hru_solrad(i)= slrext(i) * (0.23+0.48*pps)
        end if

      ENDIF

1000  continue

      if(putvar('grid',   'hru_solrad',    MAXHRU, 'real', hru_solrad)
     +  .ne.0) return

      extradrun = 0


      RETURN
      END



      SUBROUTINE EXTRAD(JDAY,AVLAT,SLP,ASP,SOLRAD,DAYLIGHT)

      integer jday

      real avlat, slp, asp, daylight, solrad
      real slrcnst, w,aday, decl, r2, slop, aspt, avlatt, sndecl, wthor
      real arg1, eqlt, delthr, dum, absum, snavlt, sndif, arg2
      real dlmnwt, dlplwt, sneqlt, arg3, wtslop, radfac, tsr, tss

!..SUBROUTINE TO COMPUTE TOTAL DAILY SOLAR IRRADIANCE, IN LANGLEYS, ON
!..A TILTED SURFACE ASSUMING NO ATMOSPHERE. SUBROUTINE COMPUTES  RADFAC,
!..IN UNITS OF HOURS. RADFAC MULTIPLIES THE SOLAR CONSTANT TO GET
!..TOTAL DAILY SOLAR IRRADIANCE. RADFAC IS A FUNCTION OF LATTITUDE,
!..DAY OF YEAR, SLOPE, AND ASPECT.
!..TO GET ENERGY/AREA FOR A DAY MULTIPLY SOLAR CONSTANT, IN UNITS OF
!..ENERGY/AREA/HOUR, BY RADFAC. (SOLAR CONSTANT IN LANGLEYS/HR=117.12)
!..(TO GET AVERAGE INSTANTANEOUS IRRADIANCE FOR THE DAY DIVIDE RADFAC
!..BY 24 AND MULTIPLY SOLAR CONSTANT.)


      SLRCNST=117.12      ! LANGLEYS PER DAY
!..EVALUATE VARIABLES, ANGLES ARE IN RADIANS.
      W=0.26179939
      ADAY=JDAY
      DECL=23.5*SIN(.017214*(284.+ADAY))
      R2=0.999847+0.001406*DECL
      SLOP=0.017453*SLP
      ASPT=0.017453*ASP
      AVLATT=0.017453*AVLAT
      DECL=DECL*0.017453
      SNDECL=ABS(DECL)/DECL

!...EQUIVALENT LATITUDE:
      ARG1=(SIN(SLOP)*COS(ASPT)*COS(AVLATT)+COS(SLOP)*SIN(AVLATT))
      EQLT=ASIN(ARG1)

!...LONGITUDINAL CORRECTION OF HOUR ANGLE
      DELTHR=ATAN((SIN(ASPT)*SIN(SLOP)/(COS(SLOP)*COS(AVLATT)-
     .       COS(ASPT)*SIN(SLOP)*SIN(AVLATT))))
      IF(ASP.GT.180.0.AND.ASP.LT.360.0) GO TO 40
      IF(DELTHR.LT.0.0) DELTHR=DELTHR+3.141592654
      GO TO 50
   40 CONTINUE
      IF(DELTHR.GT.0.0)DELTHR=DELTHR-3.141592654
   50 CONTINUE
      IF(ASP.GT.0.0) GO TO 55
      DUM=ABS(AVLAT)+SLP
      IF(DUM.LE.90.0)DELTHR=0.0
      IF(DUM.GT.90.0)DELTHR=3.141592654
   55 CONTINUE

!.. EQUIVALENT HOUR ANGLE OF SUNRISE-SUNSET-HORIZONTAL SURFACE
      ABSUM=ABS(AVLATT)+ABS(DECL)
      IF(ABSUM.LT.1.57079633) GO TO 60
      SNAVLT=ABS(AVLATT)/AVLATT
      SNDIF=ABS(SNAVLT-SNDECL)
      IF(SNDIF.LT.0.0000001) WTHOR=3.141592654
      IF(SNDIF.GE.0.0000001) WTHOR=0.0
      GO TO 70
   60 CONTINUE
      ARG2=(-TAN(AVLATT)*TAN(DECL))
      WTHOR=ACOS(ARG2)
   70 CONTINUE

!..EQUIVALENT HOUR ANGLE OF SUNRISE-SUNSET-EQUIVALENT SURFACE
      DLMNWT=DELTHR-WTHOR
      DLPLWT=DELTHR+WTHOR
      ABSUM=ABS(EQLT)+ABS(DECL)
      IF(ABSUM.LT.1.57079633)GO TO 90
      SNEQLT=ABS(EQLT)/EQLT
      SNDIF=ABS(SNEQLT-SNDECL)
      IF(SNDIF.LT.0.0000001)WTSLOP=3.141592654
      IF(SNDIF.GE.0.0000001)WTSLOP=0.0
      GO TO 95
   90 ARG3=(-TAN(EQLT)*TAN(DECL))
      WTSLOP=ACOS(ARG3)
   95 CONTINUE

!..TIMES OF SUNRISE AND SUNSET FOR EQUIVALENT SURFACE
      IF(DLMNWT.GE.WTSLOP) THEN
          RADFAC=0.0
          RETURN
          END IF
      TSR=AMAX1(DLMNWT,-WTSLOP)/W
      TSS=AMIN1(DLPLWT, WTSLOP)/W
      DAYLIGHT=(TSS-TSR)/24.0

!..COMPUTE DAILY IRRADIANCE FACTOR
      RADFAC=(TSS-TSR)*SIN(EQLT)*SIN(DECL)+(12.0/3.1416)*COS(EQLT)*
     .         COS(DECL)*(SIN(W*TSS)-SIN(W*TSR))
      RADFAC=RADFAC/R2

!..calculated daily solar radiation

      SOLRAD=RADFAC * SLRCNST

      RETURN
      END
