!***********************************************************************
!
!                     POTET_DPM.F  
!
!.....MODULE to CALCULATE POTENTIAL EVAPOTRANSPIRATION USING:
!..  EITHER JENSEN-HAISE OR PRIESTLY-TAYLOR METHODS

!.. COEFFICIENTS, TX AND CT FOR JENSEN-HAISE EQUATION
!...FOR A BLOCK. THE COEFFICIENTS ARE A FUNCTION OF LONG-TERM
!...AVERAGE JULY MIN AND MAX (TNAVG,TXAVG) TEMPERATURES (WARMEST MONTH 
!...OF THE YEAR) AND ALTITUDE (ALT).

!..PREIESTLY-TAYLOR USES NET RADIATION CALCULATED IN NETRAD_DPM.F MODULE

  
!     version: 1.0 (vaccaro/bauer)


      integer function potet_dpm(arg)

      include 'fmodules.inc'

      character*(*) arg

      integer potetdecl, potetinit, potetrun, retval

      integer nhru, nmonths, cov_type(MAXHRU)

      real   hru_elev(MAXHRU), ct(MAXHRU), tx(MAXHRU)
      real   hru_potet(MAXHRU), petmin(MAXMO)

      save  nhru, nmonths, cov_type, hru_elev
      save  ct, tx, hru_potet,  petmin

      retval = 0

      if(arg.eq.'declare') then
        retval = potetdecl(hru_potet)
       
      else if(arg.eq.'initialize') then
        retval = potetinit(nhru, nmonths, hru_elev, cov_type,
     +                                              ct,tx,petmin)

      else if(arg.eq.'run') then
        retval = potetrun(nhru,nmonths,hru_potet,ct,tx,hru_elev,
     +                                          cov_type,petmin)

      end if

!******Debug level print
      call dpint4('End of potet_dpm, retval = ', retval, 1, 2)

      potet_dpm = retval

      return
      end

!***********************************************************************
!     potetdecl: Declares variables and parameters
!               for daily potential et calculations
              
!     potetdecl - set up parameters

      integer function potetdecl(hru_potet)

      include 'fmodules.inc'

      real  hru_potet(MAXHRU)

      potetdecl = 1

      if(declvar('potet', 'hru_potet', 'nhru', MAXHRU, 'real',
     +   'Potential evapotranspiration for hru',
     +   'inches',hru_potet).ne.0) return

!..Jensen-Haise parameters (CALCULATED IN INITIALIZE)

      if(declparam('potet', 'ct',          'nhru',  'real',
     +   '0.', '0.', '600.',
     +   'Jensen-Haise PET equation coefficient',
     +   'Jensen-Haise PET equation coefficient, elevation part',
     +   '????').ne.0) return
    
      if(declparam('potet', 'tx',           'nhru',  'real',
     +   '0.', '0.', '600.',
     +   'Jensen-Haise PET equation coefficient',
     +   'Jensen-Haise PET equation coefficient, temperature part',
     +   'degrees').ne.0) return
    
!..MONTHLY VALUES (nmonths)

      if(declparam('potet', 'petmin', 'nmonths', 'real',
     +   '0.', '0.', '60.',
     +   'Minimum daily PET rate for each month',
     +   'Minimum daily PET rate for each month',
     +   'inches').ne.0) return

      if(declparam('basin', 'hru_elev', 'nhru', 'real',
     +   '0.', '-300.', '15000',
     +   'Mean elevation for each HRU',
     +   'Mean elevation for each HRU',
     +   'feet').ne.0) return

      if(declparam('basin', 'cov_type', 'nhru', 'integer',
     +   '1', '1', '31',
     +   'HRU cover type: land use/cover type, from 1-31',
     +   'HRU cover type, represents from 1-31 landuses/covers',
     +   'none').ne.0) return

      potetdecl = 0

      return
      end


!**********************************************************************
!     potetinit - get input  parameters and then  calculate J-H coefs


      integer function potetinit(nhru,nmonths,hru_elev,
     +                                   cov_type,ct,tx,petmin)

      include 'fmodules.inc'

      
      integer nhru, nmonths, i

!..Variables and parameters for this module

      integer cov_type(MAXHRU)

      real  hru_elev(MAXHRU), ct(MAXHRU), tx(MAXHRU), petmin(MAXMO)
      real  hru_tmxjuly(MAXHRU), hru_tmnjuly(MAXHRU) 

      real tnav, txav, e1, e2, ch, c1

      CHARACTER*90 BUFFER

      potetinit = 1

!..GET dimensions 

      nhru = getdim('nhru')
      if(nhru.eq.-1) return

      nmonths = getdim('nmonths')
      if(nmonths.eq.-1) return

!..GET SOME VARIABLES AND PARAMETERS
      
!..elevation for hru for both jensen-haise and priestly-taylor et

      if(getparam('basin', 'hru_elev',   MAXHRU, 'real',    hru_elev)
     +   .ne.0) return

!..landuse/cover type

      if(getparam('basin', 'cov_type',   MAXHRU, 'integer', cov_type)
     +   .ne.0) return

!..MONTHLY MINIMUM PET VALUES FOR ALL HRUS/MRUS/CELLS (nmonths)

      if(getparam('potet', 'petmin',      MAXMO,   'real',  petmin)
     +   .ne.0) return

!.....MINIMUM MONTHLY RATES FOR PET, THIS VALUE WILL CONSTRAIN CALCULATED VALUES

      WRITE(BUFFER,1000)
      CALL OPSTR(BUFFER)
      WRITE(BUFFER,1001)
      CALL OPSTR(BUFFER)
      WRITE(BUFFER,1000)
      CALL OPSTR(BUFFER)
      WRITE(BUFFER,1002)
      CALL OPSTR(BUFFER)
      WRITE(BUFFER,1003) PETMIN
      CALL OPSTR(BUFFER)

1000  FORMAT()
1001  FORMAT('  THE FOLLOWING ARE MINIMUM DAILY PET VALUES TO BE USED WH
     +EN CALCULATED PET=0.0')
1002  FORMAT('  JAN   FEB   MAR   APR   MAY   JUN   JUL   AUG   SEP   OC
     +T   NOV   DEC')
1003  FORMAT(12F6.3)

!.....CALCULATE COEFFICIENTS, TX AND CT FOR JENSEN-HAISE EQUATION
!.....FOR A BLOCK. THE COEFFICIENTS ARE A FUNCTION OF LONG-TERM
!.....AVERAGE JULY MIN AND MAX (HRU_TMNJULY,HRU_TMXJULY) TEMPERATURES (WARMEST MONTH 
!.....OF THE YEAR) AND ALTITUDE (HRU_ELEV).


      if(getvar('grid', 'hru_tmnjuly',  MAXHRU, 'real', hru_tmnjuly)
     +   .ne.0) return

      if(getvar('grid', 'hru_tmxjuly',  MAXHRU, 'real', hru_tmxjuly)
     +   .ne.0) return


      DO 10 I=1,Nhru

!.....CONVERT TO CELCIUS     

      TNAV=0.5555556*(hru_tmnjuly(I)-32.)
      TXAV=0.5555556*(hru_tmxjuly(I)-32.)

!.....CALCULATE VAPOR PRESSURES

      E1=23.38*EXP(18.1-5303.3/(TNAV+273.0))
      E2=23.38*EXP(18.1-5303.3/(TXAV+273.0))

!.....EVALUATE COEFFICIENTS: tx and then ct
 
      TX(I)=27.5 - 0.25*(E2-E1) - hru_elev(I)/1000.0

       CH=50.0/(E2-E1)
      C1=68.0 - 3.6*hru_elev(I)/1000.
10    CT(I)=1.0 / (C1+13.0*CH)

      potetinit = 0

      return
      END

! ****************************************************************************

!     potetrun - runs potet module

 
      integer function potetrun(nhru,nmonths,hru_potet,ct,tx,hru_elev,
     +                                               cov_type,petmin)

      include 'fmodules.inc'


      integer  nhru, nmonths, cov_type(MAXHRU)

      real  hru_potet(MAXHRU),  ct(MAXHRU),  tx(MAXHRU)
      real  hru_elev(MAXHRU),  tavf(MAXHRU), petmin(MAXMO)
      real  hru_netrad(MAXHRU), hru_solrad(MAXHRU)

!..temporary parameters

      integer i, mon, cov, nowtime(6)

      real alt, tav, tavc, satvp, slpvp, slrnet, prsr, hvap
      real psycnst, ratio, eeq

      potetrun = 1

      call dattim('now',   nowtime)

      mon    = nowtime(2)

!..get calculated variables for use in module

      if(getvar('netrad',   'hru_netrad',   MAXHRU, 'real', hru_netrad)
     +   .ne.0) return

      if(getvar('grid',     'hru_solrad',   MAXHRU, 'real', hru_solrad)
     +   .ne.0) return

      if(getvar('grid',     'tavf',        MAXHRU, 'real', tavf)
     +   .ne.0) return

      DO 1000 i=1,nhru

!...AVERAGE TEMPERATURE FOR 24 HOUR DAY, AND CONVERT TO CELCIUS:
      TAV =  Tavf(i)
      TAVC = 5.0 * (TAV-32.0)/9.0
!.. check if right cover type to do Priestly-Taylor, if not then do Jensen-Haise

        cov = cov_type(i)

      if (cov.le.3 .or. cov.eq.10 .or. cov.eq.13 .or. cov.eq.16) then      

!...Part of Module TO COMPUTE "EQUIVALENT" EVAPOTRANSPIRATION, EEQ (IN./DAY),
!...USING PRIESTLY-TAYLOR METHOD. THE VARIBLES ARE CALCULATED
!...USING FORMULAS GIVEN IN JENSEN, 1990.
!...INPUTS:
!          ALT - ALTITUDE IN FEET
!          hru_netrad - NET RADIATION, CAL/CM**2/DAY
!...OUTPUT:
!          EEQ - CMs OF WATER PER DAY

      alt    = hru_elev(i)
      slrnet = hru_netrad(i)

!...SATURATION VAPOR PRESSURE AT AVG TEMP, KILOPASCALS (KPA):
      SATVP = EXP((16.78*TAVC-116.9)/(TAVC+237.3))

!...SLOPE OF SATURATION VAPOR PRESSURE CURVE AT AVG TEMP, KPA/DEG(CELCIUS)
      SLPVP = 4098*SATVP/(TAVC+237.3)**2

!...ATMOSPHERIC PRESSURE FOR ALTITUDE, KPA:
      PRSR = 101.3-0.003215*ALT

!...LATENT HEAT OF VAPORIZATION AT AVG TEMP, KILOJOULES/KILOGRAM:
      HVAP = 2501-2.361*TAVC

!...PSCHOMETRIC CONSTANT AT AVG TEMP FOR ALTITUDE, KPA:
      PSYCNST = 1.6286*PRSR/HVAP

!...RATIO USED IN PRIESTLY-TAYLOR FORMULA:
      RATIO = SLPVP/(SLPVP+PSYCNST)

!...COMPUTE EEQ, CM/DAY. FIRST CONVERT HVAP TO CAL/GM:
      HVAP= HVAP*0.2389
      EEQ = RATIO*SLRNET/HVAP

!...CONVERT TO INCHES/DAY

      hru_potet(i)=EEQ/2.54

      ELSE

!..not Priestly-Taylor landuse therefore use JENSEN-HAISE PET calculation

      hru_potet(i)= ct(i) * (tav - tx(i)) * hru_solrad(i) * 0.000673

      ENDIF

      if (hru_potet(i) .lt. petmin(mon)) hru_potet(i) = petmin(mon)

1000  continue

      potetrun = 0
 
      return
      end


