!***********************************************************************
!
!                     RUNOFF_DPM.F  

!...WHEN SOIL IS FULLY OR PARTIALLY SATURATED THIS MODULE COMPUTES
!...VERTICAL INFILTRATION INTO SUBSOIL, USING CONSTANT INFILTRATION
!...RATE, AND LATERAL DISCHARGE FROM SOIL TO STREAM (RUNOFF) USING DARCY'S
!...LAW.  TRANSMISSIVITY VARIES WITH SOIL SATURATION. AS LONG AS THERE IS
!...SURFACE EXCESS, TRANS=TOTAL SOIL DEPTH * HYDRAULIC CONDUCTIVITY.
!...WHEN SOIL BEGINS TO DESATURATE AN AVERAGE TRANS, BASED ON STARTING
!...AND ENDING SATURATED THICKNESS IS USED. GRADIENT=LAND SURFACE SLOPE.
!...  1) SAT FLOW FOR CELL = TRANS*GRADIENT*(WIDTH OF TRANSECT)
!                            *(# OF TRANSECTS).
!...  2) # OF TRANSECTS = (HRU AREA)/(TRANSECT WIDTH * TRANSECT LENGTH).
!...COMBINING 1) AND 2):
!...  3) SAT FLOW FOR CELL = TRANS*GRADIENT*(CELL AREA)/TRANSECT LENGTH.
!...TO BE COMPATIBLE WITH OTHER BUDGET ITEMS, SATURATED FLOW THROUGH SOIL
!...MUST BE ALSO BE EXPRESSED IN INCHES. TOTAL SATURATED FLOW IN THE CELL
!...IS THERFORE DIVIDED BY CELL AREA AND:
!...  4) SAT FLOW FOR CELL = TRANS*GRADIENT*/TRANSECT LENGTH
!...TRANSECT LENGTH IS THE REPRESENTATIVE OR "EFFECTIVE" LENGTH FROM
!...SURFACE DRAINAGE DIVIDES TO THE NEAREST EPHEMERAL DRAINAGES.
!

!     version: 1.0 (vaccaro/bauer)


      integer function runoff_dpm(arg)

      include 'fmodules.inc'

      character*(*) arg

      integer runoffdecl, runoffinit, runoffrun, retval

     
      integer nhru, nsoils, nobs, nlayrsoil
      integer nlayer(MAXSOILS), cov_type(MAXHRU), hru_soil(MAXHRU)

      real efflngth(MAXHRU),    effslp(MAXHRU),   vksat(MAXHRU)
      real solprm(MAXSOILS),    spcyld(MAXSOILS)
      real storcurnew(MAXHRU)
      real hru_rechrge(MAXHRU), excess(MAXHRU), hru_lakestor(MAXHRU)
      real hru_rodrcy(MAXHRU), hru_roexcs(MAXHRU), hru_rototal(MAXHRU)

      save nhru, nsoils, nlayrsoil,nlayer,nobs
      save vksat,efflngth,effslp,cov_type, hru_lakestor
      save hru_soil, solprm, spcyld, excess, hru_rodrcy, hru_roexcs
      save hru_rechrge, hru_rototal, storcurnew

      retval = 0

      if(arg.eq.'declare') then
        retval = runoffdecl(hru_rodrcy,hru_roexcs,hru_rechrge,
     +                     hru_rototal,storcurnew,hru_lakestor)
      
      else if(arg.eq.'initialize') then
        retval = runoffinit(nhru,nsoils,cov_type,spcyld,vksat,
     +   nobs,hru_soil,solprm,nlayer,efflngth,effslp,nlayrsoil)

      else if(arg.eq.'run') then
        retval = runoffrun(nhru,nsoils,cov_type,spcyld,vksat,
     + nlayrsoil,nobs,hru_soil,solprm,nlayer,efflngth,effslp,
     + hru_rototal,hru_rodrcy,hru_roexcs,hru_rechrge,storcurnew,
     + hru_lakestor)

      end if

!******Debug level print
      call dpint4('End of runoff_dpm, retval = ', retval, 1, 2)

      runoff_dpm = retval

      return
      end

!***********************************************************************
!     runoffdecl: Declares 
!   


!     runoffdecl - set up parameters


      integer function runoffdecl(hru_rodrcy,hru_roexcs,hru_rechrge,
     +                          hru_rototal,storcurnew,hru_lakestor)


      include 'fmodules.inc'

      real hru_rodrcy(MAXHRU), hru_roexcs(MAXHRU),  hru_rototal(MAXHRU)
      real storcurnew(MAXHRU), hru_rechrge(MAXHRU), hru_lakestor(MAXHRU)

      runoffdecl = 1

!..declare variables for this module

      if(declvar('runoff', 'hru_rodrcy', 'nhru', MAXHRU, 'real',  
     +   'Calculated surface (darcy-flow) runoff from soils for a hru',
     +   'inches',hru_rodrcy).ne.0) return

      if(declvar('runoff', 'hru_roexcs', 'nhru', MAXHRU, 'real',  
     +   'Calculated surface runoff from water in excess of saturation',
     +   'inches',hru_roexcs).ne.0) return

      if(declvar('runoff', 'hru_rototal', 'nhru', MAXHRU, 'real',  
     +   'Total runoff for a hru from darcy flow and excess water',
     +   'inches',hru_rototal).ne.0) return

      if(declvar('runoff', 'hru_rechrge', 'nhru', MAXHRU, 'real',  
     +   'Calculated recharge-water leaving bottom of the root zone',
     +   'inches',hru_rechrge).ne.0) return

      if(declvar('runoff', 'storcurnew', 'nhru', MAXHRU, 'real',  
     +   'Modifed storage in saturated store due to runoff abstraction',
     +   'inches',storcurnew).ne.0) return

      if(declvar('runoff', 'hru_lakestor', 'nhru', MAXHRU, 'real',  
     +   'P-ET for lakes for no obs surface runoff or its = 0.0',
     +   'inches',hru_lakestor).ne.0) return

!..delcare parameters for this module

      if(declparam('runoff', 'vksat',   'nhru', 'real',
     +   '10.', '0.', '100.',
     +   'HRU vertical infiltration rate of subsoil (inch/year)',
     +   'HRU vertical infiltration rate of subsoil that recharge'//
     +   'can not exceed',
     +   'in/yr').ne.0) return

      if(declparam('runoff', 'efflngth', 'nhru', 'real',
     +   '1000.', '0.', '10000.',
     +   'One-half the average spacing between smallest drainages ',
     +   'One-half the average spacing between smallest drainage '//
     +   'channels for a HRU',
     +   'feet').ne.0) return

      if(declparam('runoff', 'effslp',  'nhru', 'real',
     +   '0.05', '0.', '.75',
     +   'Effective average slope between smallest drainage channels',
     +   'Effective average slope between smallest drainage channels'//
     +   'for efflngth of an HRU, ratio of vertical to horizontal',
     +   'none').ne.0) return

      if(declparam('soilms', 'spcyld', 'nsoils', 'real',
     +   '0.20', '0.02', '.45',
     +   'Specific yield of soils for a soil association',
     +   'Specific yield of soils for a soil association, equivalent'//
     +   ' to total effective porosity, as decimal fraction by volume',
     +   'none').ne.0) return

      if(declparam('soilms', 'solprm', 'nsoils', 'real',
     +   '5.', '0.', '500.',
     +   'Lateral permeability for a soil association, in feet/day',
     +   'Lateral permeability for a soil association, in feet/day',
     +   'feet/day').ne.0) return

      if(declparam('soilms', 'nlayer', 'nsoils', 'integer',
     +   '4', '1', '14',
     +   'Number of 6" layers for a soil association',
     +   'Number of 6" layers for a soil association, will be limited'//
     +   'by the depth of the root zone during calculations',
     +   'none').ne.0) return

      if(declparam('basin', 'hru_soil', 'nhru', 'integer',
     +   '1', '1', '200',
     +   'HRU soil type',
     +   'HRU soil type',
     +   'none').ne.0) return

      if(declparam('basin', 'cov_type', 'nhru', 'integer',
     +   '1', '1', '31',
     +   'HRU cover type: land use/cover type, from 1-31',
     +   'HRU cover type, represents from 1-31 landuses/covers',
     +   'none').ne.0) return

      runoffdecl = 0

      return
      end


!**********************************************************************
!     runoffinit - get input  parameters, calculate,  and then initialize 
!               

      integer function runoffinit(nhru,nsoils,cov_type,spcyld,vksat,
     +   nobs,hru_soil,solprm,nlayer,efflngth,effslp,nlayrsoil)

      include 'fmodules.inc'
      
      integer nhru, nsoils, nobs, nlayrsoil, i
      integer nlayer(MAXSOILS), cov_type(MAXHRU), hru_soil(MAXHRU)

      real efflngth(MAXHRU),  effslp(MAXHRU), vksat(MAXHRU)
      real solprm(MAXSOILS), spcyld(MAXSOILS)

      character*60 buffer

      runoffinit = 1

!..GET dimensions 

      nhru = getdim('nhru')
      if(nhru.eq.-1) return

      nsoils = getdim('nsoils')
      if(nsoils.eq.-1) return

      nobs = getdim('nobs')
      if(nobs.eq.-1) return

      nlayrsoil = getdim('nlayrsoil')
      if(nlayrsoil.eq.-1) return

!..vertical conductivity at bottom root zone (unit gradient assumed)

      if(getparam('runoff', 'vksat',      MAXHRU, 'real',     vksat)
     +   .ne.0) return

      if(getparam('runoff', 'efflngth',   MAXHRU, 'real',   efflngth)
     +   .ne.0) return

      if(getparam('runoff', 'effslp',     MAXHRU, 'real',     effslp)
     +   .ne.0) return

!..parameters from other modules
!..landuse/cover type and soil properties

      if(getparam('basin', 'cov_type',   MAXHRU, 'integer', cov_type)
     +   .ne.0) return

      if(getparam('soilms', 'spcyld',    MAXSOILS, 'real',    spcyld)
     +   .ne.0) return

      if(getparam('soilms', 'solprm',    MAXSOILS, 'real',    solprm)
     +   .ne.0) return

      if(getparam('soilms', 'nlayer',    MAXSOILS, 'integer', nlayer)
     +   .ne.0) return

!.. Get HRU/MRU soil information for initializing

      if(getparam('basin', 'hru_soil',   MAXHRU,  'integer', hru_soil)
     +  .ne.0) return

      WRITE(BUFFER,1000)
      CALL OPSTR(BUFFER)
      WRITE(BUFFER,1001)
      CALL OPSTR(BUFFER)
      WRITE(BUFFER,1002)
      CALL OPSTR(BUFFER)

      DO 20 I=1,nhru
      WRITE(BUFFER,1003)I,VKSAT(I),efflngth(I),effslp(I)
20    CALL OPSTR(BUFFER)
      
      WRITE(BUFFER,1000)
      CALL OPSTR(BUFFER)

!..formats

1000  FORMAT()
1001  FORMAT('  ** THE FOLLOWING ARE ATTRIBUTES FOR EACH CELL: **')
1002  FORMAT('  CELL#     VKSAT   EFLNGTH   EFFSLP')
1003  FORMAT(2X,I6,2X,F9.1,F7.0,F9.4)

      runoffinit = 0

      RETURN
      END

!***********************************************************************
! 
!     runoffrun - calculates surface runoff based on both darcy flow for
!                 the saturated store  and excess flow for the amount of 
!                 water (if any) above full saturation
!


      integer function runoffrun(nhru,nsoils,cov_type,spcyld,vksat,
     + nlayrsoil,nobs,hru_soil,solprm,nlayer,efflngth,effslp,
     + hru_rototal,hru_rodrcy,hru_roexcs,hru_rechrge,storcurnew,
     + hru_lakestor)

      include 'fmodules.inc'
     
      integer nhru, nsoils, nobs, cov_type(MAXHRU), hru_soil(MAXHRU)
      integer nlayer(MAXSOILS), nlayrsoil

      real efflngth(MAXHRU),   effslp(MAXHRU),   vksat(MAXHRU)
      real solprm(MAXSOILS),   spcyld(MAXSOILS)
      real hru_rodrcy(MAXHRU), hru_roexcs(MAXHRU), hru_rototal(MAXHRU)
      real storcurnew(MAXHRU), storcur(MAXHRU),   excess(MAXHRU)
      real hru_rechrge(MAXHRU), hru_actevsoil(MAXHRU), hru_ppt(MAXHRU)
      real  sms(MAXHRU,MAXLAYERS), storpor(MAXHRU,MAXLAYERS)
      real  sms_init(MAXHRU,MAXLAYERS), storpor_init(MAXHRU,MAXLAYERS)
      real hru_chngsm(MAXHRU), hru_lakestor(MAXHRU)

!..temp vars,parms

      integer  i, n, ns, nltot
      real vhycond, hycond, recr1, recr2, ro1, ro2, slp, elnth, trans
      real frcdy1, frcdy2, frcdy3, afac, bfac, delsat, stor, storlay


      runoffrun = 1

      if(getvar('soilms', 'excess',      MAXHRU,  'real',   excess)
     +  .ne.0) return

      if(getvar('soilms', 'storcur',     MAXHRU,  'real',  storcur)
     +  .ne.0) return

      if(getvar('soilms', 'hru_chngsm',  MAXHRU,  'real', hru_chngsm)
     +  .ne.0) return

      if(getvar('soilev', 'hru_actevsoil', MAXHRU,'real',hru_actevsoil)
     +   .ne.0) return

      if(getvar('grid',   'hru_ppt',     MAXHRU,   'real', hru_ppt)
     +   .ne.0) return

      if(getvar('soilms',  'storpor', MAXHRU*MAXLAYERS,'real', storpor)
     +   .ne.0) return

      if(getvar('soilms',  'sms',    MAXHRU*MAXLAYERS,'real',   sms)
     +   .ne.0) return 

      if(getvar('soilms',  'storpor_init', MAXHRU*MAXLAYERS, 'real',
     +   storpor_init).ne.0) return

      if(getvar('soilms',  'sms_init',MAXHRU*MAXLAYERS,'real', sms_init)
     +   .ne.0) return 

!..Start HRU loop

      DO 1000 i=1,nhru

      hru_rodrcy(i)  = 0.0
      hru_roexcs(i)  = 0.0
      hru_rechrge(i) = 0.0
      hru_rototal(i) = 0.0
      hru_lakestor(i)= 0.0
      storcurnew(i)  = 0.0

!..if water, put amount of precip not et'd in total runoff and into excess ro, go to next hru

      if(cov_type(i).eq.10) then
       hru_rototal(i)= hru_ppt(i) - hru_actevsoil(i)

       if(hru_rototal(i).lt.0.0) then
         hru_lakestor(i)= hru_rototal(i)
         hru_rototal(i) = 0.0
       end if
        
       hru_roexcs(i) = hru_rototal(i)

       go to 1000
      end if

!..if impervious hru, put amount of excess not et'd in total runoff and excess ro, go to next hru

      if(cov_type(i).eq.16) then
       hru_rototal(i)= excess(i)
       hru_roexcs(i) = excess(i)
       excess(i) = 0.0
       go to 1000
      end if

!..  SOIL IDENTIFICATION NUMBER FOR THIS BLOCK
      NS=HRU_SOIL(I)
!..  NUMBER OF 6 IN LAYERS OF SOIL
      NLTOT=NLAYER(NS)
!...SOIL and ROOT DEPTH DEPTH MUST BE AT LEAST 6 INCHES (ONE LAYER)
      IF(NLTOT.LT.1) NLTOT=1

!...IF THERE IS NO SATURATION THERE IS NO RUNOFF, calculate change in sm and update
      IF(storcur(i).LT.0.00000001) THEN
           go to 500
       END IF

!   SET THE VERTICAL INFILTRATION RATE AND THE EFFECTIVE LATERAL
!   HYDRAULIC CONDUCTIVITY OF THE SOIL COLUMN

      VHYCOND=VKSAT(i)/365.0
      HYCOND =SOLPRM(NS)*12.0

      storcurnew(i)=storcur(i)

!...ESTABLISH SLOPE AND INITIALIZE COMPONENTS OF RUNOFF AND INFILTRATION.

      SLP=effslp(i)
      ELNTH=efflngth(i)*12.0
      IF(SLP.LT.0.00000001) SLP=(NLTOT*6.0)/ELNTH
      RECR1=0.0
      RECR2=0.0
      RO1=0.0
      RO2=0.0

!...IF SOIL IS INITIALLY ONLY PARTIALLY SATURATED, RUNOFF IS COMPUTED
!...USING AVERAGE TRANSMISSSIVITY (DONE LATER IN MODULE) AND A TIME
!...OF ONE WHOLE DAY. set time and go to partially saturated section (200)

      IF(excess(i).LT.0.00000001) THEN
           FRCDY2=1.0
           GO TO 200
      END IF

!...IF SOIL IS FULLY SATURATED THERE IS PROBABLY ALSO SOME SURFACE
!...excess(i). COMPUTE TIME TO DRAIN THIS excess(i) WITH COMBINED INFILTRATION
!...AND LATERAL FLOW IN FULLY SATURATED SOIL. IF DRAIN TIME IS MORE THAN
!...ONE DAY, ASSUME THAT OVERLAND FLOW ALSO OCCURS TO DRAIN ALL excess(i),
!...AND SOIL REMAINS SATURATED AT END OF DAY.

      TRANS=HYCOND*storcurnew(i)/spcyld(ns)
      hru_rodrcy(i)=TRANS*SLP/ELNTH
      FRCDY1=excess(i)/(VHYCOND+hru_rodrcy(i))

      IF(FRCDY1.GE.1.0) THEN           !..there is runoff from excess water
          hru_rechrge(i)=VHYCOND
          hru_roexcs(i) =excess(i)-hru_rechrge(i)-hru_rodrcy(i)
          hru_rototal(i) = hru_rodrcy(i) + hru_roexcs(i)
          go to 500                    !..drained all excess, do change in sm if no obs ro
      END IF

!...ALL SURFACE excess(i) DRAINED INTO THE SOIL IN LESS THAN A DAY (FRCDY1)
!...TO REPLACE THE SATURATED MOISTURE THAT DRAINED LATERALLY
!...AND DOWNWARD. COMPUTE THESE VERTCAL AND LATERAL AMOUNTS, AND FRACTION
!...OF DAY REMAINING (FRCDY2) TO DRAIN DESATURATING SOIL.

      RECR1=FRCDY1*VHYCOND
      RO1=FRCDY1*TRANS*SLP/ELNTH
      FRCDY2=1.0-FRCDY1

!...THE FOLLOWING COMPUTES THE AMOUNT OF VERTICAL PLUS LATERAL SOIL WATER
!...DRAINAGE THAT OCCURS OVER THE REMAINING PART OF THE DAY USING AN AVERAGE
!...OF THE STARTING AND ENDING TRANSMISSIVITIES.(THESE EXPRESSIONS CAN
!...BE DERIVED BY SIMULTANEOUSLY SOLVING EXPRESSIONS FOR AVERAGE
!.. TRANSMISSIVITY IN TERMS OF DRAINAGE AMOUNT, AND DRAINAGE AMOUNT
!...IN TERMS OF TRANSMISSIVITY AND VERTICAL LEAKAGE RATE.)

200   AFAC=SLP*HYCOND/(ELNTH*spcyld(ns))
      BFAC=VHYCOND+AFAC*storcurnew(i)
      DELSAT=2.0*BFAC*FRCDY2/(2.0+AFAC*FRCDY2)
   
!...COMPUTE VERTiCAL AND HORIZONTAL COMPONENTS OF DRAINAGE AND
!...REMAINING SATURATION.

      IF(DELSAT.LE.storcurnew(i)) THEN
          RECR2=FRCDY2*VHYCOND
          TRANS=HYCOND*(2.0*storcurnew(i)-DELSAT)/(2.0*spcyld(ns))
          RO2=FRCDY2*TRANS*SLP/ELNTH
          storcurnew(i)=storcurnew(i)-RECR2-RO2

!...IF COMPUTED DRAINAGE EXCEEDS INITIAL SATURATION COMPUTE
!...TIME FOR COMPLETE DRAINAGE TO OCCUR (FRCDY3),
!...AND COMPUTE VERTICAL AND HORIZONTAL COMPONENTS USING AVERAGE
!...TRANSMISSIVITY = 1/2 INITIAL TRANSMISSIVITY (BECAUSE FINAL TRANS=0).

       ELSE
          FRCDY3=storcurnew(i)/(VHYCOND+AFAC*storcurnew(i)/2.0)
          RECR2=FRCDY3*VHYCOND
          TRANS=0.5*storcurnew(i)*HYCOND/spcyld(ns)
          RO2=FRCDY3*TRANS*SLP/ELNTH
          storcurnew(i)=0.0
       END IF


      hru_rodrcy(i)  = RO1+RO2
      hru_roexcs(i)  = 0.0
      hru_rechrge(i) = RECR1+RECR2
      hru_rototal(i) = hru_rodrcy(i) + hru_roexcs(i)

500   continue                                      !...soil moisture calculations and
                                                    !.. update saturated store if no obs ro

!...IF THERE IS STREAMFLOW DATA, COMPUTE CHANGE IN SOIL MOISTURE NOT
!...EFFECTED BY THEORETICAL RUNOFF AND SAVE SOIL AND EXCESS MOISTURE
!...IN ARRAYS FOR SUBROUTINE RUNTRU.
!...COMPUTE SOIL MOISTURE CHANGE (hru_chngsm)
 
      if(nobs.gt.0) then

         DO 5 N=1,NLTOT
5        hru_chngsm(i) = hru_chngsm(i)
     +   - storpor_init(i,N)+STORPOR(I,N)-SMS_init(i,N)+SMS(I,N)


      else

!...IF THERE IS NO STREAMFLOW DATA, COMPUTE CHANGE IN SOIL MOISTURE INCLUDING
!...THAT RESULTING FROM THEORETICAL RUNOFF, SAVE NEW SOIL MOISTURE VALUES
!...AND SET EXCESS ARRAY TO 0.0
!...DISTRIBUTE SATURATED MOISTURE TO LAYERS: only done when no obs runoff

         excess(i) = 0.0         !..by definition =0.0 when no obs ro due to it is = roexcs

         STOR   = storcurnew(i)      !..current total water in saturated store
         STORLAY= spcyld(ns)*6.0     !..maximum possible water per layer in store
         DO 10 N=NLTOT,1,-1
         storpor(i,n) = AMIN1(STORLAY,STOR)
         STOR=STOR - storpor(i,n)
10       CONTINUE
         
         STORCUR(I) = storcurnew(i)
         DO 20 N=1,NLTOT
20       hru_chngsm(i) = hru_chngsm(i)
     +   - storpor_init(i,N)+STORPOR(I,N)-SMS_init(i,N)+SMS(I,N)

      END IF

1000  continue


      if(putvar('soilms', 'excess',       MAXHRU,   'real',    excess)
     +   .ne.0) return
      if(putvar('soilms', 'hru_chngsm',   MAXHRU,   'real', hru_chngsm)
     +   .ne.0) return
      if(putvar('soilms',  'storpor', MAXHRU*MAXLAYERS, 'real', storpor)
     +   .ne.0) return
      if(putvar('soilms',   'storcur',    MAXHRU,    'real',   storcur)
     +  .ne.0) return

      runoffrun = 0

      RETURN
      END


