!***********************************************************************
!
!                     RUNTRU_DPM.F  
!
!   MODULE TO APPORTION OBSERVED STREAM DISCHARGE MINUS ESTIMATED BASEFLOW
!   TO EACH CELL. APPORTIONED NON-SURFACE WATER CELL DISCHARGE IS BASED ON
!   RATIO OF SIMULATED RUNOFF FOR THE CELL TO THE TOTAL FOR ALL NON-SURFACE
!   WATER CELLS. SOIL MOISTURE FOR THOSE CELLS IS THEN REDUCED BY AMOUNT
!   OF APPORTIONED RUNOFF. FOR SURFACE WATER CELLS THE APPORTIONED DISCHARGE
!   IS EQUATED TO THE SIMULATED RUNOFF (=PRECIP-PET=hru_roexcs(I)).

  
!     version: 1.0 (vaccaro/bauer)


      integer function runtru_dpm(arg)

      include 'fmodules.inc'

      character*(*) arg

      integer runtrudecl, runtruinit, runtrurun, retval

      integer nobs, nhru, nsoils, nlayrsoil, nbsflw
      integer nlayer(MAXSOILS), cov_type(MAXHRU), hru_soil(MAXHRU)

      real disch, bsnfac, bsnara
      real hru_obsdisch(MAXHRU)
      real surplus, deficit, chgsmrobsn, rchbsn, wght(MAXHRU)       
      real vksat(MAXHRU), spcyld(MAXSOILS)            

      save nobs, nhru, nsoils, nlayer, nlayrsoil,  nbsflw
      save surplus, deficit, chgsmrobsn, rchbsn        
      save disch, bsnfac, bsnara, hru_obsdisch
      save hru_soil, cov_type, spcyld, vksat, wght

      retval = 0

      if(arg.eq.'declare') then
        retval = runtrudecl(surplus, deficit, chgsmrobsn, rchbsn,
     +                                  bsnfac,disch,hru_obsdisch)
                                
      else if(arg.eq.'initialize') then
        retval = runtruinit(nhru,nobs,nlayer,nsoils,nbsflw,
     +        nlayrsoil,wght,hru_soil,cov_type,spcyld,vksat,
     +                              bsnara,bsnfac,hru_obsdisch)

      else if(arg.eq.'run') then
        retval = runtrurun(nhru,nobs,nlayer,nsoils,nlayrsoil,
     +  hru_soil,chgsmrobsn,surplus,deficit, disch,cov_type, rchbsn,
     +                        spcyld,vksat,bsnfac,wght,hru_obsdisch)
      
      end if

!******Debug level print
      call dpint4('End of runtru, retval = ', retval, 1, 2)

      runtru_dpm = retval

      return
      end

!***********************************************************************
!     runtrudecl: Declares variables and parameters 
!                 for allocating observed runoff-baseflow to hrus
!                 based on calculated runoff


      integer function runtrudecl(surplus, deficit, chgsmrobsn, rchbsn,
     +                                     bsnfac, disch, hru_obsdisch)


      include 'fmodules.inc'

      real surplus, deficit, chgsmrobsn, rchbsn, bsnfac
      real disch, hru_obsdisch(MAXHRU)

      runtrudecl = 1

!..RUNOFF PARAMETERS

      if(declparam('runtru', 'bsnara', 'one', 'real',
     +   '1.0', '0.01', '1e+05',
     +   'Basin area to streamgauge for runoff data',
     +   'Basin area to streamgauge for runoff data if data input'//
     +   'This area may be smaller or larger than basin_area if'//
     +   'including areas outside of drainage area',
     +   'square mile').ne.0) return

!..Variables

      if(declvar('runtru', 'hru_obsdisch', 'nhru', MAXHRU, 'real',  
     +   'Apportioned observed surface runoff based on calculated ro',
     +   'inches',hru_obsdisch).ne.0) return

      if(declvar('runtru', 'disch', 'one', 1, 'real',
     +     'Observed discharge minus baseflow',
     +     'inches',disch).ne.0) return

      if(declvar('runtru', 'bsnfac', 'one', 1, 'real',
     +   'Converts cfs to inches based on bsnara',
     +   'inch/cfs',bsnfac).ne.0) return

      if(declvar('runtru', 'surplus', 'one', 1, 'real',
     +   'Basin amount of excess not drained by observed discharge',
     +   'inches',surplus).ne.0) return

      if(declvar('runtru', 'deficit', 'one', 1, 'real',
     +   'Basin amount observed discharge not supplied by free water',
     +   'inches',deficit).ne.0) return

      if(declvar('runtru', 'chgsmrobsn', 'one', 1, 'real',
     +   'Basin change in saturated soil moist due to obs discharge',
     +   'inches',chgsmrobsn).ne.0) return

      if(declvar('runtru', 'rchbsn', 'one', 1, 'real',
     +   'Basin amount of recharge for all hrus/mrus/cells',
     +   'inches',rchbsn).ne.0) return

!..  soil parameters

      if(declparam('soilms', 'spcyld', 'nsoils', 'real',
     +   '0.20', '0.02', '.45',
     +   'Specific yield of soils for a soil association',
     +   'Specific yield of soils for a soil association, equivalent'//
     +   ' to total effective porosity, as decimal fraction by volume',
     +   'none').ne.0) return

      if(declparam('soilms', 'nlayer', 'nsoils', 'integer',
     +   '4', '1', '14',
     +   'Number of 6" layers for a soil association',
     +   'Number of 6" layers for a soil association, will be limited'//
     +   'by the depth of the root zone during calculations',
     +   'none').ne.0) return

      if(declparam('runoff', 'vksat', 'nhru', 'real',
     +   '10.', '0.', '100.',
     +   'HRU vertical infiltration rate of subsoil (inch/year)',
     +   'HRU vertical infiltration rate of subsoil that recharge'//
     +   'can not exceed',
     +   'in/yr').ne.0) return

!..basin parameters

      if(declparam('basin', 'hru_soil', 'nhru', 'integer',
     +   '1', '1', '200',
     +   'HRU soil type',
     +   'HRU soil type',
     +   'none').ne.0) return

      if(declparam('basin', 'cov_type', 'nhru', 'integer',
     +   '1', '1', '31',
     +   'HRU cover type: land use/cover type, from 1-31',
     +   'HRU cover type, represents from 1-31 landuses/covers',
     +   'none').ne.0) return


      runtrudecl = 0

      return
      end


!**********************************************************************
!     runtruinit - get input  parameters, calculate,  and then initialize 
!               

      integer function runtruinit(nhru,nobs,nlayer,nsoils,nbsflw,
     +                    nlayrsoil,wght,hru_soil,cov_type,spcyld,vksat,
     +                          bsnara,bsnfac,hru_obsdisch)

      include 'fmodules.inc'

      integer nobs, nhru, nsoils, nlayrsoil, nbsflw, i
      integer nlayer(MAXSOILS), cov_type(MAXHRU), hru_soil(MAXHRU)

      real wght(MAXHRU),   hru_obsdisch(MAXHRU)
      real vksat(MAXHRU), spcyld(MAXSOILS), bsnara, bsnfac

      character*70 buffer

      runtruinit = 1

      nobs = getdim('nobs')
      if(nobs.eq.-1) return

      nhru = getdim('nhru')
      if(nhru.eq.-1) return

      nsoils = getdim('nsoils')
      if(nsoils.eq.-1) return
           
      nlayrsoil = getdim('nlayrsoil')
      if(nlayrsoil.eq.-1) return

      nbsflw = getdim('nbsflw')
      if(nbsflw.eq.-1) return

      if(getvar('basin',   'wght',       MAXHRU,   'real',     wght)
     +   .ne.0) return

      if(getparam('runtru', 'bsnara',       1,     'real',     bsnara)
     +   .ne.0) return

      if(getparam('soilms', 'nlayer',    MAXSOILS, 'integer',  nlayer)
     +   .ne.0) return

      if(getparam('basin', 'hru_soil',   MAXHRU,  'integer', hru_soil)
     +   .ne.0) return

!..landuse/cover type

      if(getparam('basin', 'cov_type',   MAXHRU,  'integer', cov_type)
     +   .ne.0) return

      if(getparam('runoff', 'vksat',      MAXHRU,   'real',      vksat)
     +   .ne.0) return

      if(getparam('soilms', 'spcyld',    MAXSOILS,  'real',     spcyld)
     +   .ne.0) return


      bsnfac = 0.0

        do 10 i=1,nhru
10      hru_obsdisch(i) = 0.0

      if(nobs.gt.0) then
        WRITE(BUFFER,1000)
        CALL OPSTR(BUFFER)
        WRITE(BUFFER,1001)  BSNARA
        CALL OPSTR(BUFFER)
        BSNFAC=.037190083/BSNARA
      endif

1000  FORMAT()
1001  FORMAT(' DRAINAGE AREA ABOVE STREAM GAGE: BSNARA=',F7.2,' SQ MI')

      runtruinit = 0

      return
      end

!***********************************************************************
! 
!     runtrurun - Adjust basin values after allocating observed runoff minus bsflw
!

      integer function runtrurun(nhru,nobs,nlayer,nsoils,
     +  nlayrsoil,hru_soil,chgsmrobsn,surplus,deficit, disch,cov_type,
     +   rchbsn,spcyld,vksat,bsnfac,wght,hru_obsdisch)

      include 'fmodules.inc'

      integer nobs, nhru, nsoils, nlayrsoil, nbsflw
      integer nlayer(MAXSOILS), cov_type(MAXHRU), hru_soil(MAXHRU)

      real disch, bsnfac,  hru_obsdisch(MAXHRU), hru_lakestor(MAXHRU)
      real runoff(MAXOBS), bsflw(MAXBSFLW)
      real surplus, deficit, chgsmrobsn, rchbsn,  rosum       
      real vksat(MAXHRU),   spcyld(MAXSOILS)
      real storpor(MAXHRU,MAXLAYERS), storcur(MAXHRU)
      real excess(MAXHRU), hru_rechrge(MAXHRU), wght(MAXHRU)
      real hru_rototal(MAXHRU), hru_roexcs(MAXHRU)

!..temp values 

      integer ns, nltot, i, l, iyr, imo, idy, nowtime(6)
      real vhycond, excs, ro, satnew, exsat
      real stor, porlay, chgsatro

      character*100 buffer

      runtrurun = 1

!***IF NO OBSERVED RUNOFF INPUT THEN RETURN******

      if(nobs.lt.1)then
       runtrurun = 0
       return
      endif

      call dattim('now', nowtime)
      iyr    = nowtime(1)
      imo    = nowtime(2)
      idy    = nowtime(3)

!..subtract baseflow from observed runoff to obtain discharge

      if(getvar('obs',  'runoff',  MAXOBS, 'real',  runoff)
     +   .ne.0) return

       disch=runoff(1)

        if(nbsflw.gt.0) then

      if(getvar('obs',  'bsflw',  MAXBSFLW, 'real',  bsflw)
     +   .ne.0) return

        disch = disch - bsflw(1)

       endif

!..convert cfs to inches

      disch=disch*bsnfac

!..GET THE CURRENT SATURATED MOISTURE FOR LAYERS

      if(getvar('soilms',  'storpor', MAXHRU*MAXLAYERS,'real', storpor)
     +   .ne.0) return

!..Daily values from the process modules

      if(getvar('runoff',  'hru_rechrge',MAXHRU,  'real',  hru_rechrge)
     +   .ne.0) return


      if(getvar('runoff', 'hru_roexcs',  MAXHRU,  'real',  hru_roexcs)
     +  .ne.0) return


      if(getvar('runoff', 'hru_rototal',  MAXHRU, 'real',  hru_rototal)
     +  .ne.0) return

      if(getvar('runoff', 'hru_lakestor',  MAXHRU, 'real', hru_lakestor)
     +  .ne.0) return

      if(getvar('soilms',   'excess',    MAXHRU,  'real',    excess)
     +   .ne.0) return

      if(getvar('soilms',   'storcur',   MAXHRU,  'real',    storcur)
     +  .ne.0) return


!....INITIALIZE BASIN SUMS COMPUTED IN THIS SUBROUTINE.

      SURPLUS   = 0.0   ! AMOUNT OF EXCESS NOT DRAINED BY OBS. DISCHARGE
      DEFICIT   = 0.0   ! AMOUNT OF OBS. DISCHARGE NOT SUPPLIED BY "FREE" WATER
      CHGSMROBSN= 0.0   ! CHANGE IN SATURATED SOIL MOISTURE DUE TO OBS. DISCHARGE
      ROSUM     = 0.0   ! TOTAL OF SIMULATED RUNOFF FOR ALL CELLS
      RCHBSN    = 0.0   ! TOTAL RECHARGE FOR ALL CELLS

!....COMPUTE WEIGHTED AVERAGE OF SIMULATED RUNOFF FOR ALL NON-SURFACE WATER and NON-IMPERVIOUS
!....HRUS/MRUS/CELLS AND INITIALIZE CELL VALUES OF PERCOLATION/RECHARGE.

      DO 10 I=1,NHRU

      hru_rechrge(I) =0.0
      hru_obsdisch(i)=0.0

      IF(COV_TYPE(I).EQ.10) THEN   !...several cases for surface-water bodies

        if(hru_roexcs(I).lt. 0.00000001) go to 10   !..no calc ro from surface-water bodies skip
                                     !...precip-aet was set as roexcs for sw in runoff_dpm
        if(disch.lt.0.000001) then   !..obs runoff=0, so put P-AET (if any) for sw in/out storage
          hru_lakestor(i)= hru_lakestor(i)+ hru_roexcs(I)
          go to 10

        else

         hru_obsdisch(i)=hru_roexcs(I)                     !..obs ro, p-aet>0
         DISCH=DISCH-hru_obsdisch(i)*wght(i)
           if(DISCH.LT.0.0)  then                 !  sim-hru-ro maybe > tot obs ro

            WRITE(buffer,1000) 
            CALL OPSTR(BUFFER)
            WRITE(buffer,1001) IYR,imo,idy,disch,hru_obsdisch(i)
            CALL OPSTR(BUFFER)
            WRITE(buffer,1002) 
            CALL OPSTR(BUFFER)

            hru_lakestor(i)= hru_lakestor(i)+ disch
            disch = 0.0
!            STOP
           endif
         GO TO 10
        endif

       END IF                    !end surface water body check/s COV_TYPE=10
 
      IF(COV_TYPE(I).EQ.16)  THEN                   !..impervious hru, do separate calcs
       if(disch .lt. 0.00000001) then
          SURPLUS=SURPLUS+hru_rototal(i)*WGHT(I)
          go to 10
       else
         RO=hru_rototal(i) 
         hru_obsdisch(i)=DISCH * hru_rototal(i) * WGHT(I)  !..allocate obs Q based on calculated Q
         DISCH=DISCH-hru_obsdisch(i)
         excs = hru_rototal(i) - hru_obsdisch(i)
         if(excs.gt.0.0)    SURPLUS=SURPLUS+excs*WGHT(I)
         if(excs.lt.0.0)    DEFICIT=DEFICIT+excs*WGHT(I)
         go to 10
       endif
      END IF       !..end if for cov_type=16

        ROSUM=ROSUM + hru_rototal(i) * WGHT(I)

10    continue
 
!....IF SIMULATED RUNOFF (ROSUM) FROM LAND SURFACES IS ZERO THERE IS NO
!....EXCESS OR SATURATED SOIL FOR ANY NON-SURFACE WATER CELL AND THUS
!... NO "FREE" WATER TO SUPPLY OBSERVED DISCHARGE FROM LAND SURFACES.
!... PUT ALL OF OBSERVED DISCHARGE FROM LAND SURFACES INTO DEFICIT.

      IF(rosum .LT. 0.00000001) THEN
          DEFICIT=-DISCH
          runtrurun = 0
        RETURN
      END IF



!....APPORTION OBSERVED LAND-SURFACE DISCHARGE MINUS BASEFLOW AMONG
!....NON-SURFACE WATER CELLS, AND ADJUST EXCESS AND SOIL MOISTURE ACCORDINGLY.

!..LOOP (9000) for all HRUS/CELLS/MRUS--ends the run part of module.........................


      DO 9000 I=1,NHRU

      IF(COV_TYPE(I).EQ.10 .or. COV_TYPE(I).EQ.16)  GO TO 9000    !..surface water or impervious, go to next hru

      
      VHYCOND=VKSAT(I)/365.0
      NS=HRU_SOIL(I)
      NLTOT=NLAYER(NS)
      excs   = excess(i)
      satnew = storcur(i)

!..see if any saturation for this hru if not go to 9000 and next hru

      IF(satnew.LT.0.00000001)          GO TO 9000

      IF(DISCH.LT.0.00000001)             GO TO 40  !..no obs q this day, but saturated soils
                                                    !.  therefore calculate recharge
      RO=hru_rototal(i) 
      hru_obsdisch(i) = DISCH * RO/ROSUM  !..allocate obs Q based on calculated Q

!....FIRST REDUCE excs BY DISCHARGE

      excs=excs-hru_obsdisch(i)
      
!....IF excs NOT ENOUGH TO SUPPLY DISCHARGE, TAKE FROM SATURATION and reduce saturation

      IF(excs.LT.0.0) THEN
            SATNEW=SATNEW+excs
            excs=0.0
      END IF

!....IF excs + SATURATION NOT ENOUGH TO SUPPLY DISCHARGE, TALLY DEFICIT, update saturated store

      IF(SATNEW.LT.0.0) THEN
            DEFICIT=DEFICIT+SATNEW*WGHT(I)
            SATNEW=0.0
            GO TO 43
      END IF

!....IF THERE IS FREE WATER LEFT, COMPUTE hru_rechrge BASED ON INFIL. RATE
!....OF SUBSOIL.

40    EXSAT=excs+SATNEW

      IF(EXSAT.LT.0.00000001) GO TO 43
      hru_rechrge(I)=AMIN1(EXSAT,VHYCOND)        !..saturation (and excess) calculate recharge

!....REDUCE excs, THEN SATURATION, BY AMOUNT hru_rechrge. IF ANY excs
!....REMAINS, TALLY INTO SURPLUS.

      excs=excs-hru_rechrge(I)
      IF(excs.LT.0.0) THEN
           SATNEW=SATNEW+excs
           excs=0.0
        ELSE
           SURPLUS=SURPLUS+excs*WGHT(I)
      END IF

!....UPDATE (reduced due to obs surface runoff) SOIL SATURATION IN ARRAY FOR NEXT DAY.
43    STOR=SATNEW

      PORLAY=SPCYLD(NS)*6.0

      DO 45 L=NLTOT,1,-1
      STORPOR(I,L)=AMIN1(PORLAY,STOR)
45    STOR=STOR-STORPOR(I,L)

!....ACCUMULATE THE REGIONAL AVERAGES AND MONTHLY CELL BUDGET ITEMS
!....AFFECTED BY THIS SUBROUTINE.

      
      CHGSATRO   = SATNEW     - storcur(i)              !..change in saturation due to obs ro
      CHGSMROBSN = CHGSMROBSN + CHGSATRO*WGHT(I)         !..abv weighted for basin change
      RCHBSN     = RCHBSN     + hru_rechrge(I)*WGHT(I)   !..accumlate recharge for basin
      storcur(i) = SATNEW

9000  CONTINUE


!..put modified variables declared in other modules back into arrays

      if(putvar('runoff',  'hru_rechrge',  MAXHRU,  'real',hru_rechrge)
     +  .ne.0) return
      if(putvar('runoff',  'hru_lakestor',  MAXHRU, 'real',hru_lakestor)
     +  .ne.0) return
      if(putvar('soilms',  'storcur',      MAXHRU,  'real',    storcur)
     +  .ne.0) return
      if(putvar('soilms',  'storpor', MAXHRU*MAXLAYERS, 'real', storpor)
     +   .ne.0) return


1000  FORMAT('**PRECIP-EVAP FROM SURFACE-WATER CELLS EXCEEDS STREAM DISC
     +HARGE-BASEFLOW**')
1001  FORMAT('DATE=',i4,2i3,' obs-disch= ',f7.3,' ro-surwat(p-et)= ',
     +f7.3)
1002  FORMAT('HAVE SET OBS DISCH FOR REST OF CELLS =0.0 & PUT DIFF IN LA
     +KE')

      runtrurun = 0

      RETURN
      END

